@@ -1,4 +1,5 @@
 #include "adaptertrimmer.h"
+#include "matcher.h"
 
 AdapterTrimmer::AdapterTrimmer(){
 }
@@ -89,6 +90,7 @@ bool AdapterTrimmer::trimBySequence(Read* r, FilterResult* fr, string& adapterse
     else if(alen >= 8)
         start = -2;
     // we start from negative numbers since the Illumina adapter dimer usually have the first A skipped as A-tailing
+    // try exact match with hamming distance (no insertion of deletion)
     for(pos = start; pos<rlen-matchReq; pos++) {
         int cmplen = min(rlen - pos, alen);
         int allowedMismatch = cmplen/allowOneMismatchForEach;
@@ -110,6 +112,41 @@ bool AdapterTrimmer::trimBySequence(Read* r, FilterResult* fr, string& adapterse
 
     }
 
+    // if failed to exact match, we try one gap
+    // to lower computational cost, we only allow one gap, and it's much enough for short reads
+    // we try insertion in the sequence
+    bool hasInsertion = false;
+    if(!found) {
+        for(pos = 0; pos<rlen-matchReq-1; pos++) {
+            int cmplen = min(rlen - pos - 1, alen);
+            int allowedMismatch = cmplen/allowOneMismatchForEach -1;
+            bool matched = Matcher::matchWithOneInsertion(rdata, adata, cmplen, allowedMismatch);
+            if(matched) {
+                found = true;
+                hasInsertion = true;
+                //cerr << ".";
+                break;
+            }
+        }
+    }
+
+    // if failed to exact match, and failed to match with one insertion in sequence
+    // we then try deletion in the sequence
+    bool hasDeletion = false;
+    if(!found) {
+        for(pos = 0; pos<rlen-matchReq; pos++) {
+            int cmplen = min(rlen - pos, alen - 1);
+            int allowedMismatch = cmplen/allowOneMismatchForEach -1;
+            bool matched = Matcher::matchWithOneInsertion(adata, rdata, cmplen, allowedMismatch);
+            if(matched) {
+                found = true;
+                hasDeletion = true;
+                //cerr << "|";
+                break;
+            }
+        }
+    }
+
     if(found) {
         if(pos < 0) {
             string adapter = adapterseq.substr(0, alen+pos);
