@@ -16,7 +16,7 @@ public:
     ~Filter();
     int passFilter(Read* r);
     bool passLowComplexityFilter(Read* r);
-    Read* trimAndCut(Read* r, int front, int tail);
+    Read* trimAndCut(Read* r, int front, int tail, int& frontTrimmed);
     bool filterByIndex(Read* r);
     bool filterByIndex(Read* r1, Read* r2);
     static bool test();
