@@ -9,13 +9,11 @@ JsonReporter::JsonReporter(Options* opt){
 JsonReporter::~JsonReporter(){
 }
 
-void JsonReporter::setDupHist(int* dupHist, double* dupMeanGC, double dupRate) {
-    mDupHist = dupHist;
-    mDupMeanGC = dupMeanGC;
+void JsonReporter::setDup(double dupRate) {
     mDupRate = dupRate;
 }
 
-void JsonReporter::setInsertHist(long* insertHist, int insertSizePeak) {
+void JsonReporter::setInsertHist(atomic_long* insertHist, int insertSizePeak) {
     mInsertHist = insertHist;
     mInsertSizePeak = insertSizePeak;
 }
@@ -26,6 +24,14 @@ void JsonReporter::report(FilterResult* result, Stats* preStats1, Stats* postSta
     ofs.open(mOptions->jsonFile, ifstream::out);
     ofs << "{" << endl;
 
+    // sequencing info
+    string sequencingInfo  = mOptions->isPaired()?"paired end":"single end";
+    if(mOptions->isPaired()) {
+        sequencingInfo += " (" + to_string(preStats1->getCycles()) + " cycles + " + to_string(preStats2->getCycles()) + " cycles)";
+    } else {
+        sequencingInfo += " (" + to_string(preStats1->getCycles()) + " cycles)";
+    }
+
     long pre_total_reads = preStats1->getReads();
     if(preStats2)
         pre_total_reads += preStats2->getReads();
@@ -68,7 +74,8 @@ void JsonReporter::report(FilterResult* result, Stats* preStats1, Stats* postSta
 
     // summary
     ofs << "\t" << "\"summary\": {" << endl;
-
+    ofs << "\t\t" << "\"fastp_version\": \""<< FASTP_VER << "\"," << endl;
+    ofs << "\t\t" << "\"sequencing\": \""<< sequencingInfo << "\"," << endl;
     ofs << "\t\t" << "\"before_filtering\": {" << endl;
     ofs << "\t\t\t" << "\"total_reads\":" << pre_total_reads << "," << endl; 
     ofs << "\t\t\t" << "\"total_bases\":" << pre_total_bases << "," << endl; 
@@ -106,21 +113,7 @@ void JsonReporter::report(FilterResult* result, Stats* preStats1, Stats* postSta
 
     if(mOptions->duplicate.enabled) {
         ofs << "\t" << "\"duplication\": {" << endl;
-        ofs << "\t\t\"rate\": " << mDupRate << "," << endl;
-        ofs << "\t\t\"histogram\": [";
-        for(int d=1; d<mOptions->duplicate.histSize; d++) {
-            ofs << mDupHist[d];
-            if(d!=mOptions->duplicate.histSize-1)
-                ofs << ",";
-        }
-        ofs << "]," << endl;
-        ofs << "\t\t\"mean_gc\": [";
-        for(int d=1; d<mOptions->duplicate.histSize; d++) {
-            ofs << mDupMeanGC[d];
-            if(d!=mOptions->duplicate.histSize-1)
-                ofs << ",";
-        }
-        ofs << "]" << endl;
+        ofs << "\t\t\"rate\": " << mDupRate << endl;
         ofs << "\t" << "}";
         ofs << "," << endl;
     }
@@ -145,21 +138,29 @@ void JsonReporter::report(FilterResult* result, Stats* preStats1, Stats* postSta
         result -> reportAdapterJson(ofs, "\t");
     }
 
+    if(result && mOptions->polyXTrimmingEnabled()) {
+        ofs << "\t" << "\"polyx_trimming\": " ;
+        result -> reportPolyXTrimJson(ofs, "\t");
+    }
+
     if(preStats1) {
         ofs << "\t" << "\"read1_before_filtering\": " ;
         preStats1 -> reportJson(ofs, "\t");
     }
 
-    if(postStats1) {
-        ofs << "\t" << "\"read1_after_filtering\": " ;
-        postStats1 -> reportJson(ofs, "\t");
-    }
-
     if(preStats2) {
         ofs << "\t" << "\"read2_before_filtering\": " ;
         preStats2 -> reportJson(ofs, "\t");
     }
 
+    if(postStats1) {
+        string name = "read1_after_filtering";
+        if(mOptions->merge.enabled)
+            name = "merged_and_filtered";
+        ofs << "\t" << "\"" << name << "\": " ;
+        postStats1 -> reportJson(ofs, "\t");
+    }
+
     if(postStats2 && !mOptions->merge.enabled) {
         ofs << "\t" << "\"read2_after_filtering\": " ;
         postStats2 -> reportJson(ofs, "\t");
