@@ -18,6 +18,7 @@ public:
     int offset;
     int overlap_len;
     int diff;
+    bool hasGap;
 };
 
 class OverlapAnalysis{
@@ -25,8 +26,8 @@ public:
     OverlapAnalysis();
     ~OverlapAnalysis();
 
-    static OverlapResult analyze(string*  r1, string*  r2, int diffLimit, int overlapRequire, double diffPercentLimit);
-    static OverlapResult analyze(Read* r1, Read* r2, int diffLimit, int overlapRequire, double diffPercentLimit);
+    static OverlapResult analyze(string*  r1, string*  r2, int diffLimit, int overlapRequire, double diffPercentLimit, bool allowGap = false);
+    static OverlapResult analyze(Read* r1, Read* r2, int diffLimit, int overlapRequire, double diffPercentLimit, bool allowGap = false);
     static Read* merge(Read* r1, Read* r2, OverlapResult ov);
 
 public:
