@@ -13,11 +13,9 @@ using namespace std;
 
 class Read{
 public:
-	Read(string name, string seq, string strand, string quality, bool phred64=false);
-    Read(string name, Sequence seq, string strand, string quality, bool phred64=false);
-	Read(string name, string seq, string strand);
-    Read(string name, Sequence seq, string strand);
-    Read(Read &r);
+	Read(string* name, string* seq, string* strand, string* quality, bool phred64=false);
+    Read(const char* name, const char* seq, const char* strand, const char* quality, bool phred64=false);
+    ~Read();
 	void print();
     void printFile(ofstream& file);
     Read* reverseComplement();
@@ -28,6 +26,8 @@ public:
     int length();
     string toString();
     string toStringWithTag(string tag);
+    void appendToString(string* target);
+    void appendToStringWithTag(string* target, string tag);
     void resize(int len);
     void convertPhred64To33();
     void trimFront(int len);
@@ -40,11 +40,10 @@ private:
 
 
 public:
-	string mName;
-	Sequence mSeq;
-	string mStrand;
-	string mQuality;
-	bool mHasQuality;
+	string* mName;
+	string* mSeq;
+	string* mStrand;
+	string* mQuality;
 };
 
 class ReadPair{
@@ -62,4 +61,11 @@ public:
     static bool test();
 };
 
+struct ReadPack {
+    Read** data;
+    int count;
+};
+
+typedef struct ReadPack ReadPack;
+
 #endif
\ No newline at end of file
