@@ -15,26 +15,11 @@
 #include "umiprocessor.h"
 #include "writerthread.h"
 #include "duplicate.h"
+#include "singleproducersingleconsumerlist.h"
+#include "readpool.h"
 
 using namespace std;
 
-struct ReadPack {
-    Read** data;
-    int count;
-};
-
-typedef struct ReadPack ReadPack;
-
-struct ReadRepository {
-    ReadPack** packBuffer;
-    atomic_long readPos;
-    atomic_long writePos;
-    //std::mutex mtx;
-    //std::mutex readCounterMtx;
-    //std::condition_variable repoNotFull;
-    //std::condition_variable repoNotEmpty;
-};
-
 typedef struct ReadRepository ReadRepository;
 
 class SingleEndProcessor{
@@ -45,31 +30,27 @@ public:
 
 private:
     bool processSingleEnd(ReadPack* pack, ThreadConfig* config);
-    void initPackRepository();
-    void destroyPackRepository();
-    void producePack(ReadPack* pack);
-    void consumePack(ThreadConfig* config);
-    void producerTask();
-    void consumerTask(ThreadConfig* config);
+    void readerTask();
+    void processorTask(ThreadConfig* config);
     void initConfig(ThreadConfig* config);
     void initOutput();
     void closeOutput();
-    void writeTask(WriterThread* config);
+    void writerTask(WriterThread* config);
+    void recycleToPool(int tid, Read* r);
 
 private:
     Options* mOptions;
-    ReadRepository mRepo;
-    atomic_bool mProduceFinished;
+    atomic_bool mReaderFinished;
     atomic_int mFinishedThreads;
-    std::mutex mInputMtx;
-    std::mutex mOutputMtx;
     Filter* mFilter;
-    gzFile mZipFile;
-    ofstream* mOutStream;
     UmiProcessor* mUmiProcessor;
     WriterThread* mLeftWriter;
     WriterThread* mFailedWriter;
     Duplicate* mDuplicate;
+    SingleProducerSingleConsumerList<ReadPack*>** mInputLists;
+    size_t mPackReadCounter;
+    atomic_long mPackProcessedCounter;
+    ReadPool* mReadPool;
 };
 
 
