@@ -9,6 +9,7 @@
 #include "writer.h"
 #include "options.h"
 #include "filterresult.h"
+#include "singleproducersingleconsumerlist.h"
 
 using namespace std;
 
@@ -26,10 +27,6 @@ public:
 
     void initWriter(string filename1);
     void initWriter(string filename1, string filename2);
-    void initWriter(ofstream* stream);
-    void initWriter(ofstream* stream1, ofstream* stream2);
-    void initWriter(gzFile gzfile);
-    void initWriter(gzFile gzfile1, gzFile gzfile2);
 
     void addFilterResult(int result, int readNum);
     void addMergedPairs(int pairs);
@@ -42,6 +39,12 @@ public:
     bool canBeStopped();
     void cleanup();
 
+    // input list
+    void setInputList(SingleProducerSingleConsumerList<ReadPack*>* list);
+    void setInputListPair(SingleProducerSingleConsumerList<ReadPack*>* left, SingleProducerSingleConsumerList<ReadPack*>* right);
+    SingleProducerSingleConsumerList<ReadPack*>* getLeftInput(){return mLeftInputList;}
+    SingleProducerSingleConsumerList<ReadPack*>* getRightInput(){return mRightInputList;}
+
 private:
     void deleteWriter();
     void writeEmptyFilesForSplitting();
@@ -55,6 +58,8 @@ private:
     Writer* mWriter2;
     Options* mOptions;
     FilterResult* mFilterResult;
+    SingleProducerSingleConsumerList<ReadPack*>* mLeftInputList;
+    SingleProducerSingleConsumerList<ReadPack*>* mRightInputList;
 
     // for spliting output
     int mThreadId;
